﻿using HarmonyLib;
using System.Collections.Generic;
using thmsn.CraftFromAllStorage.Patches;
using Steamworks;
using System;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace thmsn.CraftFromAllStorage.Network
{
    public static class NetworkMessageProcessor
    {
        public static List<Message> waiting = new List<Message>();

        public static void ListenAndProcess()
        {
            try
            {
                if (RAPI.IsCurrentSceneMainMenu()) { return; }
                uint num;
                while (SteamNetworking.IsP2PPacketAvailable(out num, Channel.DEFAULT))
                {
                    byte[] array = new byte[num];
                    uint num2;
                    CSteamID csteamID;
                    if (SteamNetworking.ReadP2PPacket(array, num, out num2, out csteamID, Channel.DEFAULT))
                    {
                        MemoryStream serializationStream = new MemoryStream(array);
                        BinaryFormatter bf = new BinaryFormatter();
                        bf.Binder = new PreMergeToMergedDeserializationBinder();
                        Packet packet = bf.Deserialize(serializationStream) as Packet;
                        Packet_Multiple packet_Multiple;
                        if (packet.PacketType == PacketType.Single)
                        {
                            Packet_Single packet_Single = packet as Packet_Single;
                            packet_Multiple = new Packet_Multiple(packet_Single.SendType);
                            packet_Multiple.messages = new Message[]
                            {
                                packet_Single.message
                            };
                        }
                        else
                        {
                            packet_Multiple = (packet as Packet_Multiple);
                        }
                        List<Message> list = packet_Multiple.messages.ToList();
                        for (int i = 0; i < list.Count; i++)
                        {
                            Message message = list[i];
                            if (message != null)
                            {
                                if (CraftFromAllStorageMod.worldLoaded)
                                {
                                    ProcessMessage(message);
                                }
                                else
                                {
                                    waiting.Add(message);
                                }
                            }

                        }
                    }
                }
            }
            catch (System.Exception ex)
            {
                Debug.LogError($"Failed to understand a network message, make sure all players are using the same version ({CraftFromAllStorageMod.modInstance.version}) of Craft From All Storage");
                Debug.LogException(ex);
                
            }
        }

        public static void ProcessMessage(Message message)
        {
            switch (message.Type)
            {
                case Message_Storage_Small_AdditionalData.MESSAGE_TYPE:
                    Message_Storage_Small_AdditionalData.HandleAdditionalData(message as Message_Storage_Small_AdditionalData);
                    break;
                case Message_Storage_Small_AnimateOpen.MESSAGE_TYPE:
                    Message_Storage_Small_AnimateOpen.HandleNetworkMessage(message as Message_Storage_Small_AnimateOpen);
                    break;
                case Message_Storage_Small_AnimateClose.MESSAGE_TYPE:
                    Message_Storage_Small_AnimateClose.HandleNetworkMessage(message as Message_Storage_Small_AnimateClose);
                    break;
            }

            if (message.Type == Message_Storage_Small_AdditionalData.MESSAGE_TYPE)
            {
                
            }
        }

        
    }

    [HarmonyPatch(typeof(GameManager), "OnWorldRecievedLate")]
    class Patch_OnWorldRecievedLate
    {
        static void Postfix()
        {
            CraftFromAllStorageMod.worldLoaded = true;

            if (NetworkMessageProcessor.waiting.Count > 0)
            {
                foreach (var message in NetworkMessageProcessor.waiting)
                {
                    NetworkMessageProcessor.ProcessMessage(message);
                }

                NetworkMessageProcessor.waiting.Clear();
            }
        }
    }

    sealed class PreMergeToMergedDeserializationBinder : SerializationBinder
    {
        public override Type BindToType(string assemblyName, string typeName)
        {
            Type typeToDeserialize = null;
            String exeAssembly = Assembly.GetExecutingAssembly().FullName;
            typeToDeserialize = Type.GetType(String.Format("{0}, {1}", typeName, exeAssembly));
            return typeToDeserialize;
        }
    }

}